use crate::db::schema::DollProfile;
use sqlx::{pool::PoolConnection, types::chrono, Sqlite};

use super::schema::{CreateDollProfile, DollTagsDb};

pub async fn get(
	mut db: DollTagsDb,
	ident: i64,
	microchip_id: &str,
) -> sqlx::Result<Option<DollProfile>> {
	sqlx::query_as!(
		DollProfile,
		r#"
			select
			id, microchip_id,
			name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color,
			created_at as "created_at!: chrono::NaiveDateTime", updated_at as "updated_at!: Option<chrono::NaiveDateTime>"
			from doll_profiles where id = ? or microchip_id = ?
		"#,
		ident,
		microchip_id
	)
	.fetch_optional(&mut **db).await
}

pub async fn check_ids(mut db: DollTagsDb, idents: &Vec<i64>) -> sqlx::Result<Vec<i64>> {
	let idents_sql_input = idents.iter().map(|_| "?").collect::<Vec<&str>>().join(", ");

	let sql = format!(
		r#"select id from doll_profiles where id in ({})"#,
		idents_sql_input
	);
	let mut query = sqlx::query_scalar(&sql);

	for ident in idents {
		query = query.bind(ident);
	}

	query.fetch_all(&mut **db).await
}

pub async fn id_exists(
	db: &mut PoolConnection<Sqlite>,
	ident: i64,
	microchip_id: &str,
) -> sqlx::Result<bool> {
	Ok(sqlx::query!(
		"select id from doll_profiles where id = ? or microchip_id = ?",
		ident,
		microchip_id
	)
	.fetch_optional(&mut **db)
	.await?
	.is_some())
}

pub async fn create(mut db: DollTagsDb, doll: CreateDollProfile<'_>) -> sqlx::Result<i64> {
	sqlx::query!(
		r#"
			insert into doll_profiles
			(id, microchip_id, name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color)
			values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
		"#,
		doll.id,
		doll.microchip_id,
		doll.name,
		doll.pronouns,
		doll.handler_name,
		doll.handler_url,
		doll.kind,
		doll.breed,
		doll.behaviour,
		doll.description,
		doll.chassis_type,
		doll.chassis_id,
		doll.chassis_color,
	).execute(&mut **db).await?;

	Ok(doll.id)
}
