use rocket::serde::Serialize;
use rocket_db_pools::{Connection, Database};
use sqlx::types::chrono;

#[derive(Database)]
#[database("dolltags")]
pub struct DollTags(sqlx::SqlitePool);
pub type DollTagsDb = Connection<DollTags>;

// Doll Profiles stuff
#[derive(Debug, Serialize)]
#[serde(crate = "rocket::serde")]
pub struct DollProfile {
	pub id: i64,
	pub microchip_id: Option<String>,
	pub created_at: chrono::NaiveDateTime,
	pub updated_at: Option<chrono::NaiveDateTime>,

	pub name: String,
	pub pronouns: String,
	pub handler_name: String,
	pub handler_url: Option<String>,

	pub kind: Option<String>,
	pub breed: Option<String>,
	pub behaviour: Option<String>,
	pub description: Option<String>,
	pub chassis_type: Option<String>,
	pub chassis_id: Option<String>,
	pub chassis_color: Option<String>,
}

impl DollProfile {
	pub fn has_notable_traits(&self) -> bool {
		self.kind.is_some()
			|| self.breed.is_some()
			|| self.behaviour.is_some()
			|| self.chassis_type.is_some()
			|| self.chassis_id.is_some()
			|| self.chassis_color.is_some()
	}
}

#[derive(Debug)]
pub struct CreateDollProfile<'a> {
	pub id: i64,
	pub microchip_id: Option<&'a str>,

	pub name: &'a str,
	pub pronouns: &'a str,
	pub handler_name: &'a str,
	pub handler_url: Option<&'a str>,

	pub kind: Option<&'a str>,
	pub breed: Option<&'a str>,
	pub behaviour: Option<&'a str>,
	pub description: Option<&'a str>,
	pub chassis_type: Option<&'a str>,
	pub chassis_id: Option<&'a str>,
	pub chassis_color: Option<&'a str>,
}
