#[macro_use]
extern crate rocket;
use std::collections::HashMap;

use db::migrate::run_migrations;
use db::schema::DollTags;
use rocket::fairing::AdHoc;
use rocket::fs::{relative, FileServer};
use rocket_db_pools::Database;
use rocket_dyn_templates::tera::try_get_value;
use rocket_dyn_templates::Template;
use routes::{error_handlers, form, public};
use serde_json::{to_value, Value};

pub mod db;
pub mod ids;
pub mod routes;

#[launch]
fn rocket() -> _ {
	rocket::build()
		.attach(Template::custom(|engines| {
			engines
				.tera
				.register_filter("pretty_id", |v: &Value, _: &HashMap<String, Value>| {
					let value = try_get_value!("pretty_id", "value", i64, v);
					Ok(to_value(ids::id_db_to_public(value)).unwrap())
				});
		}))
		.attach(DollTags::init())
		.attach(AdHoc::try_on_ignite("SQLx migrations", run_migrations))
		.register("/", catchers![error_handlers::not_found])
		.mount("/assets", FileServer::from(relative!("/assets")))
		.mount(
			"/",
			routes![
				public::index,
				public::show_profile,
				form::register_tag::show_register,
				form::register_tag::handle_register,
			],
		)
}
