use rocket::response::{Redirect, Responder};
use rocket_dyn_templates::{context, Template};

#[catch(404)]
pub fn not_found() -> Template {
	Template::render("error/not-found", context! {})
}

#[derive(Responder)]
pub enum PageResponse {
	Page(Template),
	Redirect(Redirect),
}

impl From<Template> for PageResponse {
	fn from(value: Template) -> Self {
		PageResponse::Page(value)
	}
}

impl From<Redirect> for PageResponse {
	fn from(value: Redirect) -> Self {
		PageResponse::Redirect(value)
	}
}

#[derive(Responder)]
#[response(status = 500)]
pub struct Fail(Template);

impl From<anyhow::Error> for Fail {
	fn from(value: anyhow::Error) -> Self {
		error!("Internal error: {:?}", value);
		Fail(Template::render("error/internal", context! {}))
	}
}

impl From<sqlx::Error> for Fail {
	fn from(value: sqlx::Error) -> Self {
		error!("DB error: {:?}", value);
		Fail(Template::render("error/internal", context! {}))
	}
}

pub type RawResult<T> = Result<T, Fail>;
pub type PageResult = RawResult<PageResponse>;
